using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;
using bbiwarg.Utility;

using Emgu.CV;
using Emgu.CV.Structure;

namespace bbiwarg.Output
{
    class OutputImage
    {
        public Image<Rgb, byte> Image { get; private set; }

        public OutputImage(int width, int height)
        {
            Image = new Image<Rgb, byte>(width, height);
        }

        public Color getColotAt(int x, int y)
        {

            byte red = Image.Data[y, x, 0];
            byte green = Image.Data[y, x, 1];
            byte blue = Image.Data[y, x, 2];
            return Color.FromArgb(red, green, blue);
        }

        public void drawLineSegment(bbiwarg.Utility.LineSegment2D lineSegment, Color color, int thickness = 1)
        {
            Image.Draw(new Emgu.CV.Structure.LineSegment2D(lineSegment.P1, lineSegment.P2), new Rgb(color), thickness);
        }

        public void drawContour(Contour<Point> contour, Color color, int thickness = 1)
        {
            Image.Draw(contour, new Rgb(color), thickness);
        }

        public void drawPoints(Seq<Point> points, Color color, int thickness = 1)
        {
            Image.Draw(points, new Rgb(color), thickness);
        }

        public void drawPixel(int x, int y, Color color)
        {
            Image.Data[y, x, 0] = color.R;
            Image.Data[y, x, 1] = color.G;
            Image.Data[y, x, 2] = color.B;
        }

        public void fillCircle(int x, int y, float radius, Color color)
        {
            Image.Draw(new CircleF(new PointF(x, y), radius), new Rgb(color), 0);
        }

        public void fillRectangle(int x, int y, int width, int height, Color color) {
            Image.Draw(new Rectangle(x, y, width, height), new Rgb(color),-1);
        }

        public void drawRectangle(int x, int y, int width, int height, Color color, int thichness = 0)
        {
            Image.Draw(new Rectangle(x, y, width, height), new Rgb(color), thichness);
        }

        public void drawText(int x, int y, String text, Color color)
        {
            MCvFont font = new MCvFont(Emgu.CV.CvEnum.FONT.CV_FONT_HERSHEY_PLAIN, 1, 1);
            Image.Draw(text, ref font, new Point(x, y), new Rgb(color));
        }

        public void drawDefect(ConvexityDefect defect, Color pointColor, Color lineColor)
        {
            drawLineSegment(new Utility.LineSegment2D(defect.OuterShort, defect.Inner), lineColor);
            drawLineSegment(new Utility.LineSegment2D(defect.OuterLong, defect.Inner), lineColor);

            fillCircle(defect.Inner.IntX, defect.Inner.IntY, 2, pointColor);
            fillCircle(defect.OuterShort.IntX, defect.OuterShort.IntY, 2, pointColor);
            fillCircle(defect.OuterLong.IntX, defect.OuterLong.IntY, 2, pointColor);

        }

        public void drawImage(Image<Gray, byte> image, Color color)
        {
            if (color.R != 0)
            {
                if (color.R != byte.MaxValue)
                    Image[0] = Image[0].Or(image.Mul((float)color.R / byte.MaxValue));
                else
                    Image[0] = Image[0].Or(image);
            }
            if (color.G != 0)
            {
                if (color.G != byte.MaxValue)
                    Image[1] = Image[1].Or(image.Mul((float)color.G / byte.MaxValue));
                else
                    Image[1] = Image[1].Or(image);
            }
            if (color.B != 0)
            {
                if (color.B != byte.MaxValue)
                    Image[2] = Image[2].Or(image.Mul((float)color.B / byte.MaxValue));
                else
                    Image[2] = Image[2].Or(image);
            }
        }

        public void drawQuadrangleGrid(Quadrangle quad, Color borderColor, Color gridColor, int numRows, int numCols)
        {

            Vector2D a = quad.TopLeft;
            Vector2D b = quad.TopRight;
            Vector2D c = quad.BottomRight;
            Vector2D d = quad.BottomLeft;

            Vector2D relAB = (b - a) / numCols;
            Vector2D relDC = (c - d) / numCols;
            Vector2D relBC = (c - b) / numRows;
            Vector2D relAD = (d - a) / numRows;

            for (int i = 1; i < numCols; i++)
            {
                drawLineSegment(new bbiwarg.Utility.LineSegment2D(a + i * relAB, d + i * relDC), gridColor);
            }

            for (int i = 1; i < numRows; i++)
            {
                drawLineSegment(new bbiwarg.Utility.LineSegment2D(a + i * relAD, b + i * relBC), gridColor);
            }

            drawLineSegment(new bbiwarg.Utility.LineSegment2D(a, b), borderColor);
            drawLineSegment(new bbiwarg.Utility.LineSegment2D(b, c), borderColor);
            drawLineSegment(new bbiwarg.Utility.LineSegment2D(c, d), borderColor);
            drawLineSegment(new bbiwarg.Utility.LineSegment2D(d, a), borderColor);
        }

        public void drawBorder(Color color) {
            drawRectangle(0, 0, Image.Width-1, Image.Height-1, color);
        }

        public void drawTouchGesture(List<Vector2D> positions, float opacity=1)
        {
            Vector2D maxPixel = Parameters.ImageMaxPixel;
            int numPositions = positions.Count;
            Color lineColor = Parameters.TouchEventVisualizerLineColor;
            Color pointColor = Parameters.TouchEventVisualizerPointColor;
            Color lineColorFaded = Color.FromArgb((int)(opacity * lineColor.R), (int)(opacity * lineColor.G), (int)(opacity * lineColor.B));
            Color pointColorFaded = Color.FromArgb((int)(opacity * pointColor.R), (int)(opacity * pointColor.G), (int)(opacity * pointColor.B));
            for (int i = 1; i < numPositions; i++)
            {
                drawLineSegment(new Utility.LineSegment2D(positions[i - 1].scale(maxPixel), positions[i].scale(maxPixel)), lineColorFaded);
            }
            Vector2D lastPos = positions[numPositions - 1].scale(maxPixel);
            fillCircle(lastPos.IntX, lastPos.IntY, 3, pointColorFaded);
        }
    }
}
